/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package core;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import pedviz.algorithms.FamilySplitter;
import pedviz.algorithms.HierarchieUpDown;
import pedviz.algorithms.Highlighter;
import pedviz.algorithms.RubberBands;
import pedviz.algorithms.SameParents;
import pedviz.algorithms.filter.Filter;
import pedviz.algorithms.filter.NumberCondition;
import pedviz.algorithms.filter.TextCondition;
import pedviz.algorithms.sugiyama.RandomSplitter;
import pedviz.algorithms.sugiyama.SicknessSplitter;
import pedviz.algorithms.sugiyama.Splitter;
import pedviz.algorithms.sugiyama.SugiyamaLayout;
import pedviz.graph.Cluster;
import pedviz.graph.Graph;
import pedviz.graph.LayoutedGraph;
import pedviz.graph.Node;
import pedviz.loader.CsvGraphLoader;
import pedviz.view.DefaultEdgeView;
import pedviz.view.DefaultNodeView;
import pedviz.view.GraphView;
import pedviz.view.GraphView3D;
import pedviz.view.NodeEvent;
import pedviz.view.NodeListener;
import pedviz.view.animations.BlinkAnimation;
import pedviz.view.rules.ColorRule;
import pedviz.view.rules.GradientRule;
import pedviz.view.rules.HighlightedColorRule;
import pedviz.view.rules.Rule;
import pedviz.view.rules.ShapeRule;
import pedviz.view.symbols.SymbolQualitativeTrait;
import pedviz.view.symbols.SymbolSexFemale;
import pedviz.view.symbols.SymbolSexMale;
import pedviz.view.symbols.SymbolSexUndesignated;
import pedviz.view.symbols3d.SymbolSexFemale3d;
import pedviz.view.symbols3d.SymbolSexFemaleLow;
import pedviz.view.symbols3d.SymbolSexMale3d;
import pedviz.view.symbols3d.SymbolSexMaleLow;
import pedviz.view.symbols3d.SymbolSexUndesignated3d;
import pedviz.view.symbols3d.SymbolSexUndesignatedLow;
import ui.DialogOpen;
import ui.DialogProgress;
import ui.FrameAdapter;
import ui.panels.ControlPanel;
import ui.panels.FilterPanel;
import ui.panels.HighlightingPanel;
import ui.panels.LayoutPanel;
import ui.panels.QualitativeTraitsPanel;
import ui.panels.TraitsPanel;
import java.text.*;

public class DemoAppCore implements NodeListener, ActionListener, Events,
		ControlPanelEvents {

	private ArrayList<Graph> families;

	private FrameAdapter frame;

	private DialogOpen dialogOpen;

	private Graph currentGraph = null;

	private CsvGraphLoader loader;

	private DefaultNodeView defaultNodeView;

	private DefaultEdgeView defaultEdgeView;

	private int highlighting = Highlighter.MATERNAL;

	private Vector<Rule> rules;

	private ShapeRule barRule;

	private ColorRule colorRule;

	private HashMap<Double, Color> colors;

	private GradientRule gradientRule;

	private QualitativeTraitsPanel qtrait;

	private HighlightingPanel hpanel;

	private FilterPanel filter;

	private TraitsPanel traits;

	private boolean bar = false;

	private boolean useGradient = false;

	private Splitter splitter = new RandomSplitter();

	private LayoutPanel lpanel;

	private BlinkAnimation animation;

	private boolean useTransparents = true;

	private boolean useAntialiasing = true;

	private boolean useMaterial = true;

	private boolean useLight = true;

	private boolean useLowQuality = false;

	public DemoAppCore() {
		defaultNodeView = new DefaultNodeView();
		defaultNodeView.setSize(7);
		defaultNodeView.setColor(Color.WHITE);
		defaultNodeView.setBorderColor(Color.GRAY);
		defaultNodeView.addHintAttribute("pid");

		defaultEdgeView = new DefaultEdgeView();
		defaultEdgeView.setHighlightedColor(new Color(255, 255, 0));
		defaultEdgeView.setWidth(0.3f);
		defaultEdgeView.setConnectChildren(true);
		defaultEdgeView.setGapBottom(5);
		defaultEdgeView.setColor(Color.LIGHT_GRAY);
		defaultEdgeView.setColorForLongLines(Color.DARK_GRAY);
		defaultEdgeView.setAlphaForLongLines(0.4f);

		defineRules();
	}

	private void defineRules() {
		rules = new Vector<Rule>();
		if (useLowQuality) {
			SymbolSexFemaleLow female = new SymbolSexFemaleLow();
			rules.add(new ShapeRule("sex", "1", female));
			SymbolSexMaleLow male = new SymbolSexMaleLow();
			rules.add(new ShapeRule("sex", "2", male));
			SymbolSexUndesignatedLow other = new SymbolSexUndesignatedLow();
			rules.add(new ShapeRule("sex", "-1", other));
		} else {
			SymbolSexFemale3d female = new SymbolSexFemale3d();
			female.setStyle(SymbolSexFemale3d.STYLE_TURM);
			rules.add(new ShapeRule("sex", "1", female));
			SymbolSexMale3d male = new SymbolSexMale3d();
			male.setStyle(SymbolSexMale3d.STYLE_TURM);
			rules.add(new ShapeRule("sex", "2", male));
			SymbolSexUndesignated3d other = new SymbolSexUndesignated3d();
			other.setStyle(SymbolSexMale3d.STYLE_TURM);
			rules.add(new ShapeRule("sex", "-1", other));
		}
		rules.add(new HighlightedColorRule("sex", "1", new Color(255, 255, 0)));
		rules.add(new HighlightedColorRule("sex", "0", new Color(255, 255, 0)));
		rules.add(new ShapeRule("sex", "1", new SymbolSexMale()));
		rules.add(new ShapeRule("sex", "2", new SymbolSexFemale()));
		rules.add(new ShapeRule("sex", "-1", new SymbolSexUndesignated()));
		colorRule = new ColorRule("", "", Color.WHITE);
		colorRule.addRule("", "", Color.WHITE);
		colorRule.addRule("", "", Color.WHITE);
		colorRule.addRule("", "", Color.WHITE);
		colors = new HashMap<Double, Color>();
		gradientRule = new GradientRule("", colors);
		gradientRule.setEnabled(false);
		barRule = new ShapeRule(new SymbolQualitativeTrait("-", 0, 13, 6, 0));

		barRule.setEnabled(false);
		rules.add(colorRule);
		rules.add(gradientRule);
		rules.add(barRule);
	}

	public void setupGraphView(GraphView graphView) {
		graphView.setBackgroundColor(new Color(0, 0, 0));
		graphView.setSelectionEnabled(true);
		graphView.setMultiselection(true);
		graphView.setRules(rules);
		graphView.addNodeListener(this);
		if (graphView instanceof GraphView3D) {
			((GraphView3D) graphView).setAntialiasingEnabled(useAntialiasing);
			((GraphView3D) graphView).setTransparencyEnabled(useTransparents);
			((GraphView3D) graphView).setLightEnabled(useLight);
			((GraphView3D) graphView).setMaterialEnabled(!useLowQuality);
		}
	}

	private void setupRubberBands(RubberBands rubberBands, int mode) {
		rubberBands.setDepth(550f);
		if (mode == FrameAdapter.VIEW_2D)
			rubberBands.setHorizontalGap(3);
		rubberBands.setVerticalSpacing(80f);
	}

	public void registerFrame(FrameAdapter frame) {
		this.frame = frame;
		frame.getContent().clearFamilyTree();
		registerPanels();
		frame.setActionsEnabled(false);
	}

	private void registerPanels() {
		traits = new TraitsPanel(this);
		frame.getContent().registerPanel(traits);
		qtrait = new QualitativeTraitsPanel(this);
		qtrait
				.setColourBarEnabled(frame.getContent().getMode() != FrameAdapter.VIEW_3D);
		frame.getContent().registerPanel(qtrait);
		filter = new FilterPanel(this);
		frame.getContent().registerPanel(filter);

		hpanel = new HighlightingPanel(this);
		frame.getContent().registerPanel(hpanel);
		lpanel = new LayoutPanel(this);
		frame.getContent().registerPanel(lpanel);
	}

	private void showOpenDialog() {
		JFileChooser dialog = new JFileChooser();
		dialog.addChoosableFileFilter(new CSVFilter());
		dialog.setMultiSelectionEnabled(false);
		dialog.showOpenDialog(frame.getFrame());
		if (dialog.getSelectedFile() != null) {
			String filename = dialog.getSelectedFile().getPath();
			loader = buildLoader(filename);
			dialogOpen = new DialogOpen(frame.getFrame());
			dialogOpen.addActionListener(this);
			dialogOpen.setColData(loader.getColumns());
			dialogOpen.setFilename(filename);
			dialogOpen.setVisible(true);
		}
	}

	private CsvGraphLoader buildLoader(String filename) {
		loader = new CsvGraphLoader(filename, ",");
		if (loader.getColumns().length == 1) {
			loader = new CsvGraphLoader(filename, ";");
			while (loader.getColumns().length == 1) {
				String splitter = JOptionPane
						.showInputDialog("Please enter the split-charakter:");
				if (splitter != null) {
					loader = new CsvGraphLoader(filename, splitter);
				} else {
					return null;
				}
			}
		}
		return loader;
	}

	private void loadFile(String filename) {
		frame.setWaitCursor();
		Graph graph = new Graph();
		loader.setSettings(dialogOpen.getId(), dialogOpen.getMom(), dialogOpen
				.getDad());
		loader.load(graph);

		frame.setPedigreeName(graph.getName());
		frame.getContent().clearFamilyTree();

		DialogProgress dialog = new DialogProgress(frame.getFrame(),
				"Progress Information", "Load Pedigree from File...");

		FamilySplitter splitter = new FamilySplitter(graph);
		SameParents sameParents = new SameParents(splitter.getFamilies());
		dialog.addTask(splitter);
		dialog.addTask(sameParents);
		dialog.run();
		graph.clear();

		families = splitter.getFamilies();
		for (int i = 0; i < families.size(); i++) {
			Graph family = families.get(i);
			if (family.getAllNodes().size() >= 10) {
				family.buildHierarchie(new HierarchieUpDown());
				int count = 0;
				for (Node node: family.getAllNodes()){
					if (!node.isDummy()){
						count++;
					}
				}
				family.setName("Family_" + i + " ("
						+ count + ")");
				frame.getContent().addFamilie(family.getName(), i);
			}
		}
		frame.setDefaultCursor();
		frame.getContent().updateFamilyTree();
		frame.setActionsEnabled(true);
		frame.setGraphViewEnabled(false);
	}

	public void updatePedigree(int index) {
		updatePedigree(families.get(index));
	}

	private void updatePedigree(Graph graph) {
		if (currentGraph != graph && graph != null) {
			showGraph(graph);
			HashMap<String, ArrayList<Comparable>> allValues = new HashMap<String, ArrayList<Comparable>>();
			HashMap<String, ArrayList<Comparable>> valuesDouble = new HashMap<String, ArrayList<Comparable>>();

			for (String column : loader.getColumns()) {
				ArrayList<Comparable> v = new ArrayList<Comparable>();
				boolean doubles = false;
				for (Node node : graph.getAllNodes()) {
					if (node.getUserData(column) != null) {
						Object trait = node.getUserData(column);
						try {
							Double d = Double.parseDouble(node.getUserData(
									column).toString());
							trait = d;
							doubles = true;
						} catch (Exception ex) {							
							doubles = false;
						}
						if (!v.contains(trait)) {
							v.add((Comparable) trait);
						}
					}
				}
				Collections.sort(v);
				if (doubles) {
					valuesDouble.put(column, v);
				}
				allValues.put(column, v);
			}

			filter.setValues(allValues);
			lpanel.setValues(allValues);
			qtrait.setValues(valuesDouble);

			traits.setTraits(loader.getColumns());
			qtrait.setTraits(valuesDouble.keySet().toArray());
			if (lpanel != null)
				lpanel.setTraits(loader.getColumns());
			filter.setTraits(loader.getColumns());

		}
	}

	private void showGraph(Graph graph) {
		LayoutedGraph layoutedGrah = layout(graph, frame.getContent().getMode());
		frame.getGraphView().setGraph(layoutedGrah);
		frame.getGraphView().centerGraph();
		frame.setGraphViewEnabled(true);
		frame.setSplitEnabled(false);
		currentGraph = graph;
	}

	private void updateDetails(Node node) {
		Object[] userDataList = node.getUserDataList();
		String[][] data = new String[userDataList.length][2];
		for (int i = 0; i < userDataList.length; i++) {
			data[i][0] = (String) userDataList[i];
			data[i][1] = node.getUserData((String) userDataList[i]).toString();
		}
		frame.getContent().setDetail(data);
	}

	public void onNodeEvent(NodeEvent event) {
		switch (event.getType()) {
		case NodeEvent.MOUSE_ENTER:
			updateDetails(event.getNode());
			if (highlighting != -1) {
				frame.getGraphView().unHighlightAll();
				ArrayList<Node> nodes = Highlighter.findLineOfDescents(
						currentGraph, event.getNode(), highlighting);
				frame.getGraphView().highlight(nodes);
			} else {
				frame.getGraphView().unHighlightAll();
			}
			break;
		case NodeEvent.MOUSE_LEAVE:
			frame.getGraphView().unHighlightAll();
			break;
		case NodeEvent.SELECTED:
			frame.setSplitEnabled(true);
			break;
		}
	}

	private void exportAsCSV() {
		JFileChooser dialog = new JFileChooser();
		dialog.addChoosableFileFilter(new CSVFilter());
		dialog.setMultiSelectionEnabled(false);
		dialog.showSaveDialog(frame.getFrame());
		if (dialog.getSelectedFile() != null) {
			String filename = dialog.getSelectedFile().getPath();
			frame.setWaitCursor();
			currentGraph.exportAsCsvFile(filename);
			frame.setDefaultCursor();
			JOptionPane.showMessageDialog(frame.getFrame(), "Wrote File: "
					+ filename, "Export CSV", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	private void exportAsJPEG() {
		JFileChooser dialog = new JFileChooser();
		dialog.addChoosableFileFilter(new JPEGFilter());
		dialog.setMultiSelectionEnabled(false);
		dialog.showSaveDialog(frame.getFrame());
		if (dialog.getSelectedFile() != null) {
			String filename = dialog.getSelectedFile().getPath();
			frame.setWaitCursor();
			frame.getGraphView().exportJPEG(filename, 2f);
			frame.setDefaultCursor();
			JOptionPane.showMessageDialog(frame.getFrame(), "Wrote File: "
					+ filename, "Export JPEG", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	private LayoutedGraph layout(Graph graph, int mode) {
		frame.setWaitCursor();
		SugiyamaLayout layout = new SugiyamaLayout(graph, defaultNodeView,
				defaultEdgeView);
		if (mode == FrameAdapter.VIEW_3D) {
			layout.setSplitter(splitter);
		}
		RubberBands rubberBunds = new RubberBands(layout.getLayoutGraph());
		setupRubberBands(rubberBunds, mode);
		DialogProgress dialog = new DialogProgress(frame.getFrame(),
				"Progress Information", "Layout Pedigree...");
		dialog.addTask(layout);
		dialog.addTask(rubberBunds);
		dialog.run();
		frame.setDefaultCursor();
		return layout.getLayoutGraph();
	}

	private void addToShowcase() {
		LayoutedGraph graph = frame.getGraphView().getGraph();
		frame.getContent().addToShowcase(graph, currentGraph.getName());
	}

	public void splitSelection() {
		if (frame.getGraphView().getSelectionCount() > 1) {
			String splitname = JOptionPane
					.showInputDialog("Please enter a name for this part of the pedigree");
			if (splitname != null) {
				Cluster cluster = new Cluster();
				for (Node node : frame.getGraphView().getSelection()) {
					cluster.addNode(node);
				}
				Graph newGraph = new Graph(cluster);
				newGraph.setName(splitname);
				newGraph.buildHierarchie(new HierarchieUpDown());
				families.add(newGraph);
				frame.getContent().addSplit(splitname, families.size() - 1,
						currentGraph.getName());
				frame.getGraphView().deselect();
				frame.setSplitEnabled(false);
			}
		}
	}

	private void updateTraits(Object[][] table) {
		if (frame != null) {
			for (int i = 0; i < table.length; i++) {
				Object[] row = table[i];
				colorRule.setColor(i, (Color) row[3]);
				colorRule.setCondition(i, (String) row[1], (String) row[2]);
				colorRule.setEnabled(i, (Boolean) row[0]);
			}
			frame.getContent().updateRules();
		}
	}

	private void updateQTraits() {
		if (frame.getContent().getMode() == FrameAdapter.VIEW_2D) {
			this.bar = qtrait.isBarSelected();
			gradientRule.setEnabled(!bar);
			barRule.setEnabled(bar);
		} else {
			gradientRule.setEnabled(true);
			barRule.setEnabled(false);
		}

		if (qtrait.getMin() < qtrait.getMean()
				&& qtrait.getMean() < qtrait.getMax()) {
			colors.clear();
			colors.put(qtrait.getMin(), Color.blue);
			colors.put(qtrait.getMean(), Color.green);
			colors.put(qtrait.getMax(), Color.red);
			gradientRule.setUserdata(qtrait.getTrait());
			barRule.setShape(new SymbolQualitativeTrait(qtrait.getTrait(),
					qtrait.getMin(), qtrait.getMax(), qtrait.getMean(), 0));
			frame.getContent().updateRules();
			useGradient = true;
		} else {
			JOptionPane.showMessageDialog(frame.getFrame(),
					"Invalid values: min < mean < max", "Error",
					JOptionPane.ERROR_MESSAGE);

		}
	}

	private void resetQTraits() {
		gradientRule.setEnabled(false);
		barRule.setEnabled(false);
		frame.getContent().updateRules();
	}

	private void updateSplitter(String trait, String value) {
		if (trait.equals("random")) {
			splitter = new RandomSplitter();
		} else {
			splitter = new SicknessSplitter(new String[] { trait },
					new String[] { value });
		}
		if (currentGraph != null) {
			showGraph(currentGraph);
		}
	}

	private void updateHighlighter(int index) {
		switch (index) {
		case 0:
			this.highlighting = Highlighter.MATERNAL;
			break;
		case 1:
			this.highlighting = Highlighter.PATERNAL;
			break;
		case 2:
			this.highlighting = Highlighter.MATERNAL_AND_PATERNAL;
			break;
		case 3:
			this.highlighting = Highlighter.ANCESTORS;
			break;
		case 4:
			this.highlighting = Highlighter.SUCCESSORS;
			break;
		case 5:
			this.highlighting = Highlighter.SUCCESSORS_AND_ANCESTORS;
			break;
		case 6:
		default:
			this.highlighting = -1;
			break;
		}
	}

	private void updateFilter() {
		Filter myFilter = new Filter();
		myFilter.setOperator(filter.getSelectedMode());
		for (int i = 0; i < filter.getConditions(); i++) {
			if (!filter.getTrait(i).equals("-----")) {
				try {
					double min = Double.parseDouble(filter.getMinValue(i));
					double max = Double.parseDouble(filter.getMaxValue(i));
					myFilter.addCondition(new NumberCondition(filter
							.getTrait(i), filter.getComperator(i), min, max));

				} catch (Exception e) {
					myFilter.addCondition(new TextCondition(filter.getTrait(i),
							filter.getComperator(i), filter.getMinValue(i),
							filter.getMaxValue(i)));

				}
			}
		}

		if (animation != null)
			animation.stop();
		frame.getGraphView().showAll();

		Set<Node> test = new HashSet<Node>();
		for (Node _node : myFilter.execute(currentGraph))
			test.add(_node);

		if (filter.getMode() == 0) {
			frame.getGraphView().hideAll();
			for (Node _node : test)
				frame.getGraphView().show(_node);
		}

		if (filter.getMode() == 1) {
			animation = new BlinkAnimation(frame.getGraphView(), test,
					Color.RED, Color.RED);
			animation.setInterval(500);
			animation.start();
		}
	}

	private void resetFilter() {
		if (animation != null)
			animation.stop();
		frame.getGraphView().showAll();
	}

	public void updatePressed(ControlPanel sender) {
		if (sender == qtrait) {
			updateQTraits();
		}

		if (sender == hpanel) {
			updateHighlighter(hpanel.getSelectedMode());
		}

		if (sender == traits) {
			updateTraits(traits.getRows());
		}

		/*
		 * if (sender == lpanel) { updateSplitter(lpanel.get]); }
		 */

		if (sender == filter) {
			updateFilter();
		}
	}

	public void resetPressed(ControlPanel sender) {
		if (sender == qtrait) {
			resetQTraits();
		}

		if (sender == filter) {
			resetFilter();
		}
	}

	public void actionPerformed(ActionEvent e) {
		Actions action = Actions.valueOf(e.getActionCommand());

		switch (action) {
		case OPEN_DIALOG_OK:
			dialogOpen.setVisible(false);
			loadFile(dialogOpen.getFilename());
			break;

		case OPEN_DIALOG_CANCEL:
			dialogOpen.setVisible(false);
			break;

		case OPEN_FILE:
			showOpenDialog();
			break;

		case SPLIT_PEDIGREE:
			splitSelection();
			break;

		case SHOW_EDGES:
			boolean visible = frame.getGraphView().isEdgeVisible();
			frame.getGraphView().setEdgeVisible(!visible);
			break;

		case CENTER_GRAPH:
			frame.getContent().resetSliders();
			frame.getGraphView().centerGraph();
			frame.updateGraphView();
			break;

		case ADD_TO_SHOWCASE:
			addToShowcase();
			break;

		case EXPORT_CSV:
			exportAsCSV();
			break;

		case EXPORT_JPEG:
			exportAsJPEG();
			break;

		case CLEAR_SELECTION:
			frame.getGraphView().deselect();
			frame.setSplitEnabled(false);
			break;

		case HELP_CONTENTS:
			break;

		case ABOUT_DIALOG:
			break;

		case CLOSE:
			System.exit(0);
			break;
		case ROTATE_X:
			if (frame.getContent().getMode() != FrameAdapter.VIEW_2D) {
				if (frame.getGraphView() != null) {
					JCheckBoxMenuItem s = (JCheckBoxMenuItem) e.getSource();
					((GraphView3D) (frame.getGraphView()))
							.setXRotatingEnabled(s.isSelected());
				}
			}
			break;
		case ROTATE_Y:
			if (frame.getContent().getMode() != FrameAdapter.VIEW_2D) {
				if (frame.getGraphView() != null) {
					JCheckBoxMenuItem s = (JCheckBoxMenuItem) e.getSource();
					((GraphView3D) (frame.getGraphView()))
							.setYRotatingEnabled(s.isSelected());
				}
			}
			break;
		default:
		case UNKNOW_COMMAND:
			JOptionPane.showMessageDialog(frame.getFrame(), "comming soon...",
					"comming soon...", JOptionPane.ERROR_MESSAGE);
		}
	}

	class CSVFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return f.getName().toLowerCase().endsWith(".csv");
		}

		public String getDescription() {
			return "CSVs";
		}
	};

	class JPEGFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return f.getName().toLowerCase().endsWith(".jpg");
		}

		public String getDescription() {
			return "JPEGs";
		}
	}

	public boolean isUseAntialiasing() {
		return useAntialiasing;
	}

	public void setUseAntialiasing(boolean useAntialiasing) {
		this.useAntialiasing = useAntialiasing;
	}

	public boolean isUseLight() {
		return useLight;
	}

	public void setUseLight(boolean useLight) {
		this.useLight = useLight;
	}

	public boolean isUseLowQuality() {
		return useLowQuality;
	}

	public void setUseLowQuality(boolean useLowQuality) {
		this.useLowQuality = useLowQuality;
		defineRules();
	}

	public boolean isUseMaterial() {
		return useMaterial;
	}

	public void setUseMaterial(boolean useMaterial) {
		this.useMaterial = useMaterial;
	}

	public boolean isUseTransparents() {
		return useTransparents;
	}

	public void setUseTransparents(boolean useTransparents) {
		this.useTransparents = useTransparents;
	};

}
