/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Panel;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;

import pedviz.graph.LayoutedGraph;
import pedviz.view.GraphView;
import pedviz.view.GraphView2D;
import pedviz.view.GraphView3D;
import ui.panels.ControlPanel;
import ui.showcase.Showcase;
import ui.showcase.ShowcaseItem;
import ui.showcase.ShowcaseListener;
import ui.table.TreeNodeData;

import com.sun.j3d.utils.universe.SimpleUniverse;

import core.Events;

public class ContentPanel extends JPanel implements ShowcaseListener {
	private GraphView graphView;

	private Showcase showcase;

	private JScrollBar moveXSlider, moveYSlider;

	private JTabbedPane mainPanel;

	private JTable detailsTable;

	private JScrollPane detailsScrollPane;

	private JTree familiesTree;

	private JScrollPane familiesScollPane;

	private JPanel showcasePanel, leftPanel, rightPanel;

	private ControlPanel top, bottom;

	private Vector<ControlPanel> panels;

	private Events listener;

	private DefaultTableModel detailsTableModel;

	private DefaultTreeModel familiesTreeModel;

	private DefaultMutableTreeNode rootTreeNode;

	private String pedigreeName = "noname";

	private int mode = FrameAdapter.VIEW_2D;

	private Panel graphPanel;

	public ContentPanel(ActionListener actionListener, Events listener, int mode) {
		this.listener = listener;
		this.mode = mode;
		if (mode == FrameAdapter.VIEW_2D) {
			graphView = new GraphView2D();
			showcase = new Showcase<GraphView2D>();
		} else {
			graphView = new GraphView3D(SimpleUniverse
					.getPreferredConfiguration());
			showcase = new Showcase<GraphView3D>();
		}
		listener.setupGraphView(graphView);
		showcase.setListener(this);
		panels = new Vector<ControlPanel>();
		buildGui();
	}

	public void registerPanel(ControlPanel panel) {
		if (panel.getVizMode() == mode
				|| panel.getVizMode() == ControlPanel.BOOTH) {
			panels.add(panel);
			rightPanel.add(panel);
		}
	}

	private void buildGui() {
		setLayout(new BorderLayout());
		// --------------- Left ----------------------
		leftPanel = new JPanel();
		leftPanel.setPreferredSize(new Dimension(170, 170));
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
		leftPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		{
			// ------------- Families -----------------
			familiesScollPane = new JScrollPane();
			familiesTree = new JTree();
			familiesScollPane.setViewportView(familiesTree);
			familiesTree.setModel(familiesTreeModel);
			familiesTree.addTreeSelectionListener(new TreeHandler());
			familiesTree.setCellRenderer(new TreeRenderer());
			top = new ControlPanel("Families", null);
			top.addInput(familiesScollPane, true);
			leftPanel.add(top);
			// ------------- Details ------------------
			detailsScrollPane = new JScrollPane();
			detailsTable = new JTable();
			detailsTableModel = new DefaultTableModel();
			detailsScrollPane.setViewportView(detailsTable);
			detailsTable.setModel(detailsTableModel);
			bottom = new ControlPanel("Details", null);
			bottom.addInput(detailsScrollPane, true);
			leftPanel.add(bottom);
		}

		// ------------------------ Center -------------------------
		mainPanel = new JTabbedPane();

		{
			graphPanel = new Panel();
			graphPanel.setLayout(new BorderLayout());
			mainPanel.addTab("Pedigree", null, graphPanel, null);
			showcasePanel = new JPanel();
			showcasePanel.setLayout(new BorderLayout());
			mainPanel.addTab("Showcase (0)", showcasePanel);
			mainPanel.setEnabledAt(1, false);
			if (mode == FrameAdapter.VIEW_3D) {

				moveYSlider = new JScrollBar(JScrollBar.HORIZONTAL);
				graphPanel.add(moveYSlider, BorderLayout.SOUTH);
				moveYSlider.setValue(0);
				moveYSlider.setMaximum(95);
				moveYSlider.setMinimum(-95);
				moveYSlider.addAdjustmentListener(new AdjustmentListener() {
					public void adjustmentValueChanged(AdjustmentEvent arg0) {
						graphView.setTranslateX(moveYSlider.getValue() / 70f);
					}
				});

				moveXSlider = new JScrollBar(JScrollBar.VERTICAL);
				graphPanel.add(moveXSlider, BorderLayout.EAST);
				moveXSlider.setValue(0);
				moveXSlider.setMaximum(135);
				moveXSlider.setMinimum(-135);
				moveXSlider.addAdjustmentListener(new AdjustmentListener() {
					public void adjustmentValueChanged(AdjustmentEvent arg0) {
						graphView.setTranslateY(moveXSlider.getValue() / 70f);
					}
				});
			}
			graphPanel.add(graphView.getComponent(), BorderLayout.CENTER);

		}

		// --------------- Right ----------------------
		rightPanel = new JPanel();
		rightPanel.setPreferredSize(new Dimension(170, 170));
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
		rightPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		rightPanel.add(Box.createVerticalGlue());
		add(leftPanel, BorderLayout.WEST);
		add(mainPanel, BorderLayout.CENTER);
		add(rightPanel, BorderLayout.EAST);
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		leftPanel.setEnabled(enabled);
		graphView.getComponent().setEnabled(enabled);
		familiesTree.setEnabled(enabled);
		detailsTable.setEnabled(enabled);
		bottom.setEnabled(enabled);
		top.setEnabled(enabled);
		if (moveXSlider != null) {
			moveXSlider.setEnabled(enabled);
		}
		if (moveYSlider != null) {
			moveYSlider.setEnabled(enabled);
		}
		for (ControlPanel panel : panels) {
			panel.setEnabled(enabled);
		}
	}

	public void setGraphViewEnabled(boolean enabled) {
		if (moveXSlider != null) {
			moveXSlider.setEnabled(enabled);
		}
		if (moveYSlider != null) {
			moveYSlider.setEnabled(enabled);
		}
	}

	public void clearFamilyTree() {
		familiesTree.removeAll();
		if (!pedigreeName.equals("noname")) {
			rootTreeNode = new DefaultMutableTreeNode();
			rootTreeNode.setUserObject(new TreeNodeData(pedigreeName, -1));
			familiesTreeModel = new DefaultTreeModel(rootTreeNode);
			familiesTree.setModel(familiesTreeModel);
		}
	}

	public void setShowcaseTitle(String title) {
		mainPanel.setTitleAt(1, title);
	}

	public void updateShowcase(int index) {
		setShowcaseTitle("Showcase (" + showcase.getShowcaseSize() + " )");
		mainPanel.setEnabledAt(1, showcase.getShowcaseSize() > 0);
	}

	public void addToShowcase(LayoutedGraph graph, String name) {
		GraphView canvas = null;
		if (mode == FrameAdapter.VIEW_2D)
			canvas = new GraphView2D();
		else
			canvas = new GraphView3D(SimpleUniverse.getPreferredConfiguration());
		listener.setupGraphView(canvas);
		canvas.setGraph(graph);
		showcase.addGraphView(canvas, name);
		canvas.centerGraph();
	}

	public Showcase getShowcase() {
		return showcase;
	}

	public void setDetail(String[][] data) {
		detailsTableModel.setDataVector(data, new String[] { "Key", "Value" });
	}

	public void addFamilie(String caption, int id) {
		DefaultMutableTreeNode child = new DefaultMutableTreeNode();
		TreeNodeData data = new TreeNodeData(caption, id);
		child.setUserObject(data);
		rootTreeNode.add(child);
	}

	public void updateFamilyTree() {
		familiesTreeModel.nodeStructureChanged(rootTreeNode);
	}

	public void addSplit(String caption, int id, String parentName) {
		DefaultMutableTreeNode child = new DefaultMutableTreeNode();
		TreeNodeData data = new TreeNodeData(caption, id);
		child.setUserObject(data);
		for (int i = 0; i < rootTreeNode.getChildCount(); i++) {
			DefaultMutableTreeNode parent = (DefaultMutableTreeNode) rootTreeNode
					.getChildAt(i);
			if (parent.toString().equals(parentName)) {
				parent.add(child);
				familiesTreeModel.nodeStructureChanged(parent);
			}
		}
	}

	public GraphView getGraphView() {
		return graphView;
	}

	public void setPedigreeName(String name) {
		pedigreeName = name;
	}

	public void updateRules() {
		graphView.updateRules();
		for (Object item : showcase.getGraphViews()) {
			((ShowcaseItem) item).getGraphView().updateRules();
		}
	}

	class TreeHandler implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent evt) {
			if (familiesTree.getSelectionPath() != null) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) familiesTree
						.getSelectionPath().getLastPathComponent();

				TreeNodeData data = (TreeNodeData) node.getUserObject();
				if (data != null && data.getId() > -1) {
					if (listener != null) {
						listener.updatePedigree(data.getId());
						graphView.centerGraph();
						graphView.updateGraphView();
					}
				}
			}
		}
	}

	class TreeRenderer extends DefaultTreeCellRenderer {
		ImageIcon root = new ImageIcon("res/csvfile.png");

		ImageIcon pedigree = new ImageIcon("res/pedigree.png");

		public TreeRenderer() {
			super();
		}

		public Component getTreeCellRendererComponent(JTree tree, Object value,
				boolean sel, boolean expanded, boolean leaf, int row,
				boolean hasFocus) {

			super.getTreeCellRendererComponent(tree, value, sel, expanded,
					leaf, row, hasFocus);
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
			TreeNodeData data = (TreeNodeData) node.getUserObject();
			setIconTextGap(8);
			if (data.getId() != -1) {
				setIcon(pedigree);
			} else {
				setIcon(root);
			}

			return this;
		}
	}

	public int getMode() {
		return mode;
	}

	public void resetSliders() {
		if (moveXSlider != null) {
			moveXSlider.setValue(0);
		}
		if (moveYSlider != null) {
			moveYSlider.setValue(0);
		}
	}
}
