/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.border.EmptyBorder;

import core.Actions;

public class DialogOpen extends JDialog {
	private String defaultId = "pid";

	private String defaultSex = "sex";

	private String defaultMom = "mom";

	private String defaultDad = "dad";

	private DefaultComboBoxModel idModel;

	private DefaultComboBoxModel sexModel;

	private DefaultComboBoxModel momModel;

	private DefaultComboBoxModel dadModel;

	private String filename;

	private JButton okButton;

	private JButton cancleButton;

	private JComboBox idComboBox;

	private JComboBox sexComboBox;

	private JComboBox momComboBox;

	private JComboBox dadComboBox;

	public DialogOpen(Frame parent) {
		this(parent, null);
	}

	public DialogOpen(Frame parent, String splitter) {
		super(parent, "Import-Settings", true);

		JPanel container = new JPanel();
		container.setBorder(new EmptyBorder(new Insets(12, 12, 11, 11)));
		container.setLayout(new GridLayout(5, 2, 10, 10));
		getContentPane().add(container);

		container.add(new JLabel("Id"));
		idComboBox = new JComboBox();
		idModel = new DefaultComboBoxModel();
		idComboBox.setModel(idModel);
		container.add(idComboBox);

		container.add(new JLabel("Sex"));
		sexComboBox = new JComboBox();
		sexModel = new DefaultComboBoxModel();
		sexComboBox.setModel(sexModel);
		container.add(sexComboBox);

		container.add(new JLabel("Mom"));
		momComboBox = new JComboBox();
		momModel = new DefaultComboBoxModel();
		momComboBox.setModel(momModel);
		container.add(momComboBox);

		container.add(new JLabel("Dad"));
		dadComboBox = new JComboBox();
		dadModel = new DefaultComboBoxModel();
		dadComboBox.setModel(dadModel);
		container.add(dadComboBox);

		okButton = new JButton("Load File");
		cancleButton = new JButton("Cancel");

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BorderLayout());
		buttonPanel.add(new JSeparator(), BorderLayout.NORTH);

		Box buttonBox = new Box(BoxLayout.X_AXIS);
		buttonBox.setBorder(new EmptyBorder(new Insets(5, 10, 5, 10)));
		buttonBox.add(okButton);
		buttonBox.add(Box.createHorizontalStrut(5));
		buttonBox.add(cancleButton);
		buttonPanel.add(buttonBox, java.awt.BorderLayout.EAST);

		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		setSize(350, 230);
		setResizable(false);
		setLocationRelativeTo(parent);
	}

	public void addActionListener(ActionListener l) {
		okButton.addActionListener(l);
		okButton.setActionCommand(Actions.OPEN_DIALOG_OK.toString());
		cancleButton.addActionListener(l);
		cancleButton.setActionCommand(Actions.OPEN_DIALOG_CANCEL.toString());
	}

	public void setColData(String[] cols) {
		idModel.removeAllElements();
		sexModel.removeAllElements();
		momModel.removeAllElements();
		dadModel.removeAllElements();

		for (String col : cols) {
			idModel.addElement(col);
			sexModel.addElement(col);
			momModel.addElement(col);
			dadModel.addElement(col);
		}
		idModel.setSelectedItem(defaultId);
		sexModel.setSelectedItem(defaultSex);
		momModel.setSelectedItem(defaultMom);
		dadModel.setSelectedItem(defaultDad);
	}

	public String getFilename() {
		return filename;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public String getId() {
		return (String) idModel.getSelectedItem();
	}

	public String getSex() {
		return (String) sexModel.getSelectedItem();
	}

	public String getMom() {
		return (String) momModel.getSelectedItem();
	}

	public String getDad() {
		return (String) dadModel.getSelectedItem();
	}

}
