/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui;

import java.awt.Cursor;
import java.awt.Frame;

import javax.swing.JApplet;
import javax.swing.SwingUtilities;

import pedviz.view.GraphView;
import core.DemoAppCore;
import core.Events;

/**
 * @author Luki
 */
public class FrameApplet extends JApplet implements FrameAdapter {

	private MainMenu mainMenu;

	private ContentPanel content;

	private Events mainWindowListener;

	private int mode;

	public FrameApplet(int mode) {
		super();
		this.mode = mode;
	}

	public void init() {
		DemoAppCore demoAppCore = new DemoAppCore();
		buildGui(demoAppCore);
		setEventListener(demoAppCore);
	}

	public void buildGui(DemoAppCore demoAppCore) {
		mainMenu = new MainMenu(demoAppCore, true);
		setJMenuBar(mainMenu);
		content = new ContentPanel(demoAppCore, demoAppCore, mode);
		getContentPane().add(content);
	}

	public void setEventListener(Events mainWindowListener) {
		this.mainWindowListener = mainWindowListener;
		this.mainWindowListener.registerFrame(this);
	}

	public void updateGraphView() {
		content.getGraphView().updateGraphView();
	}

	public GraphView getGraphView() {
		return content.getGraphView();
	}

	public void setActionsEnabled(boolean status) {
		content.setEnabled(status);
		mainMenu.setEnabled(status);
	}

	public void setGraphViewEnabled(boolean status) {
		content.setGraphViewEnabled(status);
		mainMenu.setGraphViewEnabled(status);
	}

	public void setSplitEnabled(boolean status) {
		mainMenu.setSplitEnabled(status);
	}

	public void setPedigreeName(String name) {
		content.setPedigreeName(name);
	}

	public Frame getFrame() {
		return (Frame) SwingUtilities.getAncestorOfClass(Frame.class, this);
	}

	public ContentPanel getContent() {
		return content;
	}

	public void setWaitCursor() {
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	}

	public void setDefaultCursor() {
		setCursor(null);
	}
}
