/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui;

import java.awt.Cursor;
import java.awt.Frame;
import java.awt.event.ActionListener;

import javax.swing.*;

import pedviz.view.GraphView;
import core.Events;

/**
 * @author Luki
 */
public class FrameDesktop extends JFrame implements FrameAdapter {

	private MainMenu mainMenu;

	private ContentPanel content;

	private Events mainWindowListener;

	private String pedigreeName = "noname";

	public FrameDesktop(ActionListener actionListener, Events listener, int mode) {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		if (mode == VIEW_3D) {
			setTitle("3DPedViz ---- Mode: 2,5d");
		} else {
			setTitle("3DPedViz ---- Mode: 2d");
		}
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		mainMenu = new MainMenu(actionListener,mode == VIEW_3D);
		setJMenuBar(mainMenu);
		content = new ContentPanel(actionListener, listener, mode);
		getContentPane().add(content);
		this.pack();
		this.setSize(800, 600);
	}

	public void setEventListener(Events mainWindowListener) {
		this.mainWindowListener = mainWindowListener;
		this.mainWindowListener.registerFrame(this);
	}

	public void updateGraphView() {
		content.getGraphView().updateGraphView();
	}

	public GraphView getGraphView() {
		return content.getGraphView();
	}

	public void setActionsEnabled(boolean status) {
		content.setEnabled(status);
		mainMenu.setEnabled(status);
	}

	public void setGraphViewEnabled(boolean status) {
		content.setGraphViewEnabled(status);
		mainMenu.setGraphViewEnabled(status);
	}

	public void setSplitEnabled(boolean status) {
		mainMenu.setSplitEnabled(status);
	}

	public void setPedigreeName(String name) {
		content.setPedigreeName(name);
		setTitle("3dPedViz - " + pedigreeName);
	}

	public Frame getFrame() {
		return this;
	}

	public void setWaitCursor() {
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	}

	public void setDefaultCursor() {
		setCursor(null);
	}

	public ContentPanel getContent() {
		return content;
	}

}
