/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;

import core.Actions;

public class MainMenu extends JMenuBar {
	private JMenuItem aboutMenuItem;

	private JSeparator jSeparator_IL6;

	private JMenuItem helpMenuItem;

	private JMenu helpMenu;

	private JMenuItem exportMenuItem;

	private JMenuItem centerGraphMenuItem;

	private JSeparator jSeparator_IL5;

	private JSeparator jSeparator_IL4;

	private JCheckBoxMenuItem showEdgesMenuItem;

	private JMenu viewMenu;

	private JSeparator jSeparator_IL3;

	private JMenuItem saveCsvMenuItem;

	private JSeparator jSeparator_IL2;

	private JMenuItem addMenuItem;

	private JSeparator jSeparator_IL1;

	private JMenuItem clearMenuItem;

	private JMenuItem splitMenuItem;

	private JMenu editMenu;

	private JMenuItem exitMenuItem;

	private JSeparator jSeparator_IL;

	private JMenuItem fileMenuItem;

	private JMenu fileMenu;

	public MainMenu(ActionListener actionListener, boolean mode3d) {
		// ----------------- Main-Menu --------------------------//
		{

			fileMenu = new JMenu();
			add(fileMenu);
			fileMenu.setText("File");
			{
				fileMenuItem = new JMenuItem();
				fileMenu.add(fileMenuItem);
				fileMenuItem.setText("Pedigree-File...");
				fileMenuItem.setActionCommand(Actions.OPEN_FILE.toString());
				fileMenuItem.addActionListener(actionListener);
			}
			{
				jSeparator_IL = new JSeparator();
				fileMenu.add(jSeparator_IL);
			}
			{
				exitMenuItem = new JMenuItem();
				fileMenu.add(exitMenuItem);
				exitMenuItem.setText("Exit");
				exitMenuItem.setActionCommand(Actions.CLOSE.toString());
				exitMenuItem.addActionListener(actionListener);
			}
		}
		{
			editMenu = new JMenu();
			add(editMenu);
			editMenu.setText("Edit");
			{
				splitMenuItem = new JMenuItem();
				editMenu.add(splitMenuItem);
				splitMenuItem.setText("Split Selection");
				splitMenuItem.setActionCommand(Actions.SPLIT_PEDIGREE
						.toString());
				splitMenuItem.addActionListener(actionListener);
			}
			{
				clearMenuItem = new JMenuItem();
				editMenu.add(clearMenuItem);
				clearMenuItem.setText("Clear Selection");
				clearMenuItem.setActionCommand(Actions.CLEAR_SELECTION
						.toString());
				clearMenuItem.addActionListener(actionListener);
			}
			{
				jSeparator_IL1 = new JSeparator();
				editMenu.add(jSeparator_IL1);
			}
			{
				addMenuItem = new JMenuItem();
				editMenu.add(addMenuItem);
				addMenuItem.setText("Add to Showcase");
				addMenuItem
						.setActionCommand(Actions.ADD_TO_SHOWCASE.toString());
				addMenuItem.addActionListener(actionListener);
			}
			{
				jSeparator_IL2 = new JSeparator();
				editMenu.add(jSeparator_IL2);
			}
			{
				saveCsvMenuItem = new JMenuItem();
				editMenu.add(saveCsvMenuItem);
				saveCsvMenuItem.setText("Save as csv...");
				saveCsvMenuItem.setActionCommand(Actions.EXPORT_CSV.toString());
				saveCsvMenuItem.addActionListener(actionListener);
			}
			{
				jSeparator_IL3 = new JSeparator();
				editMenu.add(jSeparator_IL3);
			}
		}
		{
			viewMenu = new JMenu();

			viewMenu.setText("View");
			add(viewMenu);
			{
				centerGraphMenuItem = new JMenuItem();
				viewMenu.add(centerGraphMenuItem);
				centerGraphMenuItem.setText("Center Graph");
				centerGraphMenuItem.setActionCommand(Actions.CENTER_GRAPH
						.toString());
				centerGraphMenuItem.addActionListener(actionListener);
			}
			{
				jSeparator_IL4 = new JSeparator();
				viewMenu.add(jSeparator_IL4);
			}
			{
				showEdgesMenuItem = new JCheckBoxMenuItem();
				viewMenu.add(showEdgesMenuItem);
				showEdgesMenuItem.setText("Show Edges");
				showEdgesMenuItem.setSelected(true);
				showEdgesMenuItem.setActionCommand(Actions.SHOW_EDGES
						.toString());
				showEdgesMenuItem.addActionListener(actionListener);
			}
			if (mode3d) {
				{
					/*
					 * colorsMenuItem = new JMenuItem();
					 * viewMenu.add(colorsMenuItem);
					 * colorsMenuItem.setText("Colors...");
					 * colorsMenuItem.setActionCommand(CommandPool.COLORS_DIALOG
					 * .toString());
					 * colorsMenuItem.addActionListener(actionListener);
					 */
				}
				{
					jSeparator_IL5 = new JSeparator();
					viewMenu.add(jSeparator_IL5);
				}
				{
					JMenuItem xRotating = new JCheckBoxMenuItem();
					viewMenu.add(xRotating);
					xRotating.setText("Rotation X");
					xRotating.setActionCommand(Actions.ROTATE_X.toString());
					xRotating.addActionListener(actionListener);
				}
				{
					JMenuItem yRotating = new JCheckBoxMenuItem();
					viewMenu.add(yRotating);
					yRotating.setText("Rotation Y");
					yRotating.setActionCommand(Actions.ROTATE_Y.toString());
					yRotating.addActionListener(actionListener);
					yRotating.setSelected(true);
				}
			}
			{
				jSeparator_IL5 = new JSeparator();
				viewMenu.add(jSeparator_IL5);
			}
			{
				exportMenuItem = new JMenuItem();
				viewMenu.add(exportMenuItem);
				exportMenuItem.setText("Export as JPEG...");
				exportMenuItem.setActionCommand(Actions.EXPORT_JPEG.toString());
				exportMenuItem.addActionListener(actionListener);
			}
		}
		{
			helpMenu = new JMenu();
			add(helpMenu);
			helpMenu.setText("About");
			/*
			 * { helpMenuItem = new JMenuItem(); helpMenu.add(helpMenuItem);
			 * helpMenuItem.setText("Help Contents"); } { jSeparator_IL6 = new
			 * JSeparator(); helpMenu.add(jSeparator_IL6); }
			 */
			{
				aboutMenuItem = new JMenuItem();
				helpMenu.add(aboutMenuItem);
				aboutMenuItem.setText("About 3DPedViz");
				aboutMenuItem.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent arg0) {
						JDialog f = new DialogAbout(new JFrame());
						f.show();
					}

				});
			}
		}

	}

	public void setEnabled(boolean enabled) {
		splitMenuItem.setEnabled(enabled);
		addMenuItem.setEnabled(enabled);
		saveCsvMenuItem.setEnabled(enabled);
		clearMenuItem.setEnabled(enabled);
		showEdgesMenuItem.setEnabled(enabled);
		exportMenuItem.setEnabled(enabled);
		// colorsMenuItem.setEnabled(enabled);
	}

	public void setSplitEnabled(boolean enabled) {
		splitMenuItem.setEnabled(enabled);
	}

	public void setGraphViewEnabled(boolean enabled) {
		splitMenuItem.setEnabled(enabled);
		addMenuItem.setEnabled(enabled);
		saveCsvMenuItem.setEnabled(enabled);
		clearMenuItem.setEnabled(enabled);
		showEdgesMenuItem.setEnabled(enabled);
		exportMenuItem.setEnabled(enabled);
		// colorsMenuItem.setEnabled(status);
	}
}
