/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui.panels;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import core.ControlPanelEvents;

public class ControlPanel extends JPanel implements ActionListener {

	private int count = 0;

	private GridBagConstraints clabel;

	private GridBagConstraints t, f;

	private GridBagConstraints cinput;

	protected ControlPanelEvents listener;

	private JLabel label;

	private String title;

	private JPanel content;

	private Box buttonBox;

	protected JButton updateButton, resetButton;

	public static final int ONLY_2D = 2;

	public static final int ONLY_3D = 4;

	public static final int BOOTH = -1;

	private int show = BOOTH;

	private boolean half = false;

	public ControlPanel(String title, ControlPanelEvents listener,
			boolean buttons, boolean half) {
		this.title = title;
		this.listener = listener;
		this.half = half;
		setLayout(new BorderLayout());
		label = new JLabel(title);
		label.setHorizontalAlignment(JLabel.CENTER);
		add(label, BorderLayout.NORTH);
		content = new JPanel();
		add(content, BorderLayout.CENTER);
		buttonBox = new Box(BoxLayout.X_AXIS);
		buttonBox.setBorder(new EmptyBorder(new Insets(5, 10, 5, 10)));
		if (buttons) {
			updateButton = new JButton("Update");
			updateButton.setActionCommand("update");
			updateButton.addActionListener(this);
			resetButton = new JButton("Reset");
			resetButton.setActionCommand("reset");
			resetButton.addActionListener(this);
			buttonBox.add(updateButton);
			buttonBox.add(Box.createHorizontalStrut(5));
			buttonBox.add(resetButton);
		}
		add(buttonBox, BorderLayout.SOUTH);
		if (!half) {
			content.setLayout(new GridBagLayout());
		} else {
			content.setLayout(new GridLayout(6, 2));
		}
		EtchedBorder border = new EtchedBorder(1);
		setBorder(border);
		clabel = new GridBagConstraints();
		clabel.fill = GridBagConstraints.HORIZONTAL;
		clabel.insets = new Insets(2, 2, 2, 2);
		clabel.gridx = 0;
		clabel.gridy = 1;
		if (!half) {
			clabel.weightx = 0.5;
		}
		cinput = new GridBagConstraints();
		cinput.fill = GridBagConstraints.HORIZONTAL;
		cinput.insets = new Insets(2, 2, 2, 2);
		cinput.gridx = 1;
		cinput.gridy = 1;
		if (!half) {
			cinput.weightx = 1.0;
		} else {
			cinput.weightx = 0.5;
		}
		t = new GridBagConstraints();
		t.fill = GridBagConstraints.HORIZONTAL;
		t.insets = new Insets(2, 2, 2, 2);
		t.gridx = 0;
		t.gridy = 0;
		t.gridwidth = 2;
		t.weightx = 1.0;
		f = new GridBagConstraints();
		f.fill = GridBagConstraints.BOTH;
		f.insets = new Insets(0, 0, 0, 0);
		f.gridx = 0;
		f.gridy = 1;
		f.gridwidth = 2;
		f.weightx = 1.0;
		f.weighty = 1.0;
	}

	public ControlPanel(String title, ControlPanelEvents listener) {
		this(title, listener, false, false);
	}

	public ControlPanel(String title, ControlPanelEvents listener,
			boolean buttons) {
		this(title, listener, buttons, false);
	}

	public JLabel addLabel(String string) {
		if (!half) {
			count++;
		}
		JLabel label = new JLabel(string);
		if (!half) {
			content.add(label, clabel);
			clabel.gridy = cinput.gridy + 1;
		} else {
			content.add(label);
		}
		return label;
	}

	public void addInput(Component component, boolean full) {
		if (!full)
			addInput(component);
		else {
			content.add(component, f);
		}
	}

	public JPanel addDummy() {
		count++;
		JPanel dummy = new JPanel();
		if (!half) {
			content.add(dummy, clabel);
		} else {
			content.add(dummy);
		}
		clabel.gridy = cinput.gridy + 1;
		return dummy;
	}

	public void addInput(Component component) {
		count++;
		if (!half) {
			if (count % 2 == 0) {
				content.add(component, cinput);
				cinput.gridy = cinput.gridy + 1;
			} else {
				content.add(component, clabel);
				clabel.gridy = clabel.gridy + 1;
			}
		} else {
			content.add(component);
		}
	}

	public String getTitle() {
		return title;
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		label.setEnabled(enabled);
		if (updateButton != null) {
			updateButton.setEnabled(enabled);
		}
		if (resetButton != null) {
			resetButton.setEnabled(enabled);
		}
	}

	public void addButton(JButton button) {
		buttonBox.add(Box.createHorizontalStrut(5));
		buttonBox.add(button);
	}

	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("update")) {
			listener.updatePressed(this);
		}
		if (event.getActionCommand().equals("reset")) {
			listener.resetPressed(this);
		}
	}

	public int getVizMode() {
		return show;
	}

	public void setVizMode(int mode) {
		this.show = mode;
	}
	
	public JPanel getContentPanel(){
		return content;
	}
}
