/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui.panels;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

import pedviz.algorithms.filter.Filter;
import core.ControlPanelEvents;

public class FilterPanel extends ControlPanel implements ItemListener {

	DefaultComboBoxModel[] traitModel;

	JComboBox[] traitList;

	JComboBox[] mode;

	JComboBox highmode;

	JRadioButton r1;

	JRadioButton r0;

	int n = 2;

	JLabel labelMode;
	
	private JComboBox[] minField, maxField;

	private HashMap<String, ArrayList<Comparable>> values;

	String[] modes = new String[] { "equals", "not equals", "greater",
			"lesser", "between", "not between" };

	public FilterPanel(ControlPanelEvents events) {
		super("Filter", events, true, true);
		labelMode = addLabel("Mode");
		highmode = new JComboBox(new String[] { "Trasparents", "Blinking" });
		addInput(highmode);

		traitModel = new DefaultComboBoxModel[n];
		traitList = new JComboBox[n];
		mode = new JComboBox[n];
		minField = new JComboBox[n];
		maxField = new JComboBox[n];
		for (int i = 0; i < n; i++) {
			traitModel[i] = new DefaultComboBoxModel();
			traitList[i] = new JComboBox();
			traitList[i].setEditable(false);
			traitList[i].setModel(traitModel[i]);
			traitList[i].addItemListener(this);
			addInput(traitList[i]);
			mode[i] = new JComboBox(modes);
			mode[i].addItemListener(this);
			addInput(mode[i]);
			minField[i] = new JComboBox();
			minField[i] = new JComboBox(new String[] {});
			minField[i].setEditable(true);
			maxField[i] = new JComboBox();
			maxField[i].setEditable(true);
			maxField[i] = new JComboBox(new String[] {});
			addInput(minField[i]);
			addInput(maxField[i]);
			maxField[i].setVisible(false);
			if (i < n - 1) {
				ButtonGroup group2 = new ButtonGroup();
				r0 = new JRadioButton("or", true);
				r1 = new JRadioButton("and", false);
				group2.add(r0);
				group2.add(r1);
				addInput(r0);
				addInput(r1);
			}
		}
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		for (int i = 0; i < n; i++) {
			traitList[i].setEnabled(enabled);
			mode[i].setEnabled(enabled);
			minField[i].setEnabled(enabled);
			maxField[i].setEnabled(enabled);
		}
		r1.setEnabled(enabled);
		r0.setEnabled(enabled);
		highmode.setEnabled(enabled);
		labelMode.setEnabled(enabled);
	}

	public void itemStateChanged(ItemEvent e) {
		JComboBox selectedChoice = (JComboBox) e.getSource();
		for (int i = 0; i < n; i++) {
			if (selectedChoice.equals(mode[i])) {
				if (selectedChoice.getSelectedIndex() > 3) {
					maxField[i].setVisible(true);
					updateUI();
				} else {
					maxField[i].setVisible(false);
					updateUI();
				}
			}

			if (selectedChoice.equals(traitList[i])
					&& traitModel[i].getSelectedItem() != null) {
				minField[i].removeAllItems();
				maxField[i].removeAllItems();
				if (values != null) {
					if (values.get(traitModel[i].getSelectedItem().toString()) != null)
						for (Object o : values.get(traitModel[i]
								.getSelectedItem().toString())) {
							minField[i].addItem(o.toString());
							maxField[i].addItem(o.toString());
						}
				}
			}
		}

	}

	public void setTraits(String[] traits) {
		String empty = "-----";
		for (int j = 0; j < n; j++) {
			traitModel[j].removeAllElements();
			traitModel[j].addElement(empty);
			for (int i = 0; i < traits.length; i++) {
				traitModel[j].addElement(traits[i]);
			}
		}
	}

	public void setValues(HashMap<String, ArrayList<Comparable>> values) {
		this.values = values;
	}

	public int getSelectedMode() {
		return r0.isSelected() ? Filter.OR : Filter.AND;
	}

	public String getTrait(int index) {
		return traitModel[index].getSelectedItem().toString();
	}

	public int getComperator(int index) {
		return mode[index].getSelectedIndex();
	}

	public String getMinValue(int index) {
		return minField[index].getSelectedItem().toString();
	}

	public String getMaxValue(int index) {
		return maxField[index].getSelectedItem().toString();
	}

	public int getConditions() {
		return n;
	}

	public void setConditions(int n) {
		this.n = n;
	}

	public int getMode() {
		return highmode.getSelectedIndex();
	}
}
