/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui.panels;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import core.ControlPanelEvents;

public class LayoutPanel extends ControlPanel implements ItemListener {

	private String[] hierarchies = { "UpDown", "DownUp" };

	private JLabel labelSplitter, labelValue;

	private JComboBox valueField;

	private JComboBox traitsList;

	private DefaultComboBoxModel traitModel;

	private HashMap<String, ArrayList<Comparable>> values;

	public LayoutPanel(ControlPanelEvents events) {
		super("Layout Algorithm", events);
		setVizMode(ONLY_3D);
		new JComboBox(hierarchies);
		// label = addLabel("Hierarchie");
		// addInput(hierarchieList);
		traitModel = new DefaultComboBoxModel();
		traitsList = new JComboBox();
		traitsList.addItemListener(this);
		traitsList.setModel(traitModel);
		labelSplitter = addLabel("split data");
		addInput(traitsList);
		labelValue = addLabel("Value");
		valueField = new JComboBox();
		addInput(valueField);
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		// hierarchieList.setEnabled(enabled);
		// label.setEnabled(enabled);
		valueField.setEnabled(enabled);
		traitsList.setEnabled(enabled);
		labelSplitter.setEnabled(enabled);
		labelValue.setEnabled(enabled);
	}

	public void itemStateChanged(ItemEvent e) {
		if (traitsList.getSelectedItem() != null) {
			valueField.removeAllItems();
			if (traitsList.getSelectedIndex() == 0) {
				valueField.setEnabled(false);
			} else {
				valueField.setEnabled(true);
				if (values != null) {
					String trait = traitsList.getSelectedItem().toString();
					if (values.get(trait) != null)
						for (Object o : values.get(trait)) {
							valueField.addItem(o.toString());
						}
				}
			}
			listener.updatePressed(this);
		}
	}

	public void setTraits(String[] traits) {
		String empty = "random";
		traitModel.removeAllElements();
		traitModel.addElement(empty);
		for (int i = 0; i < traits.length; i++) {
			traitModel.addElement(traits[i]);
		}
	}

	public void setValues(HashMap<String, ArrayList<Comparable>> values) {
		this.values = values;
	}
}
