/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui.panels;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableColumn;

import ui.table.ColorEditor;
import ui.table.ColorRenderer;
import ui.table.ListEditor;
import ui.table.ListRenderer;
import ui.table.TraitTableModel;
import core.ControlPanelEvents;

public class TraitsPanel extends ControlPanel implements PropertyChangeListener {

	private JScrollPane scrollPane;

	private TraitTableModel mappingModel;

	private JTable traitsTable;

	public TraitsPanel(ControlPanelEvents events) {
		super("Taits", events);
		scrollPane = new JScrollPane();
		traitsTable = new JTable();
		traitsTable.setDefaultRenderer(Color.class, new ColorRenderer(true));
		traitsTable.setDefaultEditor(Color.class, new ColorEditor());
		mappingModel = new TraitTableModel();
		traitsTable.setShowGrid(true);
		traitsTable.setModel(mappingModel);
		traitsTable.addPropertyChangeListener(this);
		scrollPane.setViewportView(traitsTable);
		addInput(scrollPane, true);
	}

	public void propertyChange(PropertyChangeEvent arg0) {
		listener.updatePressed(this);
	}

	public void setTraits(String[] traits) {
		String empty = "-----";
		Object[][] data = { { new Boolean(false), empty, "0", Color.red },
				{ new Boolean(false), empty, "0", Color.blue },
				{ new Boolean(false), empty, "0", Color.yellow },
				{ new Boolean(false), empty, "0", Color.green } };
		mappingModel.setData(data);
		traitsTable.setRowHeight(20);
		TableColumn col = traitsTable.getColumnModel().getColumn(1);
		col.setWidth(50);
		col.setMinWidth(50);
		String[] values = new String[traits.length + 1];
		values[0] = empty;
		for (int i = 0; i < traits.length; i++) {
			values[i + 1] = traits[i];
		}
		col.setCellEditor(new ListEditor(values));
		col.setCellRenderer(new ListRenderer(values));
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		traitsTable.setEnabled(enabled);
	}

	public Object[][] getRows() {
		return mappingModel.getData();
	}
}
