/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package ui.showcase;

import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.JPanel;

import pedviz.view.GraphView;

public class Showcase<o extends GraphView> extends JPanel implements
		MouseListener {
	private ArrayList<ShowcaseItem> items;

	private ShowcaseListener listener;

	private GridLayout gridLayout;

	public Showcase() {
		super();
		gridLayout = new GridLayout();
		gridLayout.setHgap(5);
		gridLayout.setVgap(0);
		setLayout(gridLayout);
		items = new ArrayList<ShowcaseItem>();
	}

	public void removeItem(int index) {
		ShowcaseItem item = items.get(index);
		items.remove(item);
		remove(item);
	}

	public void addGraphView(o graphview, String caption) {
		ShowcaseItem item = new ShowcaseItem(graphview, caption, this);
		items.add(item);
		add(item);

		int columns = (int) Math.round(Math.sqrt(items.size()));
		if (columns > 0) {
			gridLayout.setColumns(columns);
			gridLayout.setRows(0);
			setLayout(gridLayout);
		}
		if (listener != null)
			listener.updateShowcase(-1);
	}

	public int getShowcaseSize() {
		return items.size();
	}

	public ArrayList<ShowcaseItem> getGraphViews() {
		return (ArrayList<ShowcaseItem>) items;
	}

	public void mouseExited(MouseEvent arg0) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseReleased(MouseEvent arg0) {
	}

	public void mousePressed(MouseEvent arg0) {
	}

	public void mouseClicked(MouseEvent arg0) {
		ShowcaseItem item = ((ShowcaseItem) (arg0.getComponent().getParent()
				.getParent()));
		int index = items.indexOf(item);
		removeItem(index);

		int columns = (int) Math.round(Math.sqrt(items.size()));
		if (columns > 0) {
			gridLayout.setColumns(columns);
			gridLayout.setRows(0);
			setLayout(gridLayout);
		}
		updateUI();

		if (listener != null)
			listener.updateShowcase(index);
	}

	public void setListener(ShowcaseListener listener) {
		this.listener = listener;
	}

}
